# -*- coding: utf-8 -*-
"""
    * vcf-tools
    * Created by liuxiaodong on 2022/10/16.
    * Change Activity:
    *     liuxiaodong   2022/10/16.
"""

import uuid
# import wmi


class Hardware:
    @staticmethod
    def get_cpu_sn():
        """
        获取CPU序列号
        :return: CPU序列号
        """
        c = wmi.WMI()
        for cpu in c.Win32_Processor():
            # print(cpu.ProcessorId.strip())
            return cpu.ProcessorId.strip()

    @staticmethod
    def get_baseboard_sn():
        """
        获取主板序列号
        :return: 主板序列号
        """
        c = wmi.WMI()
        for board_id in c.Win32_BaseBoard():
            # print(board_id.SerialNumber)
            return board_id.SerialNumber

    @staticmethod
    def get_bios_sn():
        """
        获取BIOS序列号
        :return: BIOS序列号
        """
        c = wmi.WMI()
        for bios_id in c.Win32_BIOS():
            # print(bios_id.SerialNumber.strip)
            return bios_id.SerialNumber.strip()

    @staticmethod
    def get_disk_sn():
        """
        获取硬盘序列号
        :return: 硬盘序列号列表
        """
        c = wmi.WMI()

        disk_sn_list = []
        for physical_disk in c.Win32_DiskDrive():
            # print(physical_disk.SerialNumber)
            # print(physical_disk.SerialNumber.replace(" ", ""))
            disk_sn_list.append(physical_disk.SerialNumber.replace(" ", ""))
        return disk_sn_list

    @staticmethod
    def get_mac():
        """
        获取mac地址
        """
        mac=uuid.UUID(int = uuid.getnode()).hex[-12:]
        return ":".join([mac[e:e+2] for e in range(0,11,2)])

    @staticmethod
    def get_info():
        # cpu_sn = Hardware.get_cpu_sn()
        # baseboard_sn = Hardware.get_baseboard_sn()
        # bios_sn = Hardware.get_bios_sn()
        # disk_sn = Hardware.get_disk_sn()
        mac = Hardware.get_mac()
        return {
            # 'cpu_sn': cpu_sn,
            # 'baseboard_sn': baseboard_sn,
            # 'bios_sn': bios_sn,
            # 'disk_sn': disk_sn,
            'mac': mac
        }



from license_manager.license import License
from license_manager.utils import Hardware

def create(privateStorePath: str, privateStorePass: str, subject: str = 'castiron', consumerType: str = 'customer', consumerAmount: int = 1, description: str = '', issuedTime: str = None, expiryTime: str = None, params: dict = None, output: str = None):
    '''
    生成 license 证书

    Arguments:
        privateStorePath {str}  -- 私钥库地址
        privateStorePass {str}  -- 私钥库密码
        subject {str}           -- license 证书主题，默认 castiron
        consumerType {str}      -- license 证书类型，默认 customer
        consumerAmount {int}    -- license 证书授权客户数量，默认 1
        description {str}       -- license 证书描述信息
        issuedTime {str}        -- license 证书生效起始时间
        expiryTime {str}        -- license 证书生效结束时间
        params {str}            -- license 验证的附加参数
        output {str}            -- license 生成后输出路径文件名
    '''
    license = License(privateStorePath=privateStorePath, privateStorePass=privateStorePass)
    license.create(
        subject=subject,
        consumerType=consumerType,
        consumerAmount=consumerAmount,
        description=description,
        issuedTime=issuedTime,
        expiryTime=expiryTime,
        params=params,
        output=output
    )

def verify(publicStorePath: str, publicStorePass: str, licensePath: str, params: dict = None):
    '''
    验证 license 证书

    Arguments:
        publicStorePath {str}   -- 公钥库地址
        publicStorePass {str}   -- 公钥库密码
        licensePath {str}       -- license 证书地址
        params {str}            -- license 验证的附加参数, mac 地址验证参数不传递，验证工具会自动获取
    '''
    license = License(publicStorePath=publicStorePath, publicStorePass=publicStorePass)
    license.install(licensePath, params=params)
    print('license 证书验证通过')

def hardware():
    '''
    获取当前机器硬件信息
    '''
    hardwar_info = Hardware.get_info()
    print('\n')
    for k, v in hardwar_info.items():
        print(f'\t{k}:\t{v}')
    print('\n')


# -*- coding: utf-8 -*-
"""
    * vcf-tools
    * Created by liuxiaodong on 2022/10/16.
    * Change Activity:
    *     liuxiaodong   2022/10/16.
"""

import fire
from license_manager.command import create, verify, hardware


def main():
    fire.Fire({
        'create': create,
        'verify': verify,
        'hardware': hardware
    })


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['license_manager']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.4.0,<0.5.0', 'pycryptodome>=3.15.0,<4.0.0']

entry_points = \
{'console_scripts': ['qxlicense = license_manager.main:main']}

setup_kwargs = {
    'name': 'license-manager',
    'version': '0.2.2',
    'description': '',
    'long_description': None,
    'author': 'leaf',
    'author_email': 'ncoder@126.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""
    * vcf-tools
    * Created by liuxiaodong on 2022/10/16.
    * Change Activity:
    *     liuxiaodong   2022/10/16.
"""

import json
import base64
import re
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from Crypto.Signature import PKCS1_v1_5 as PKCS1_signature
from datetime import datetime
from license_manager.utils import Hardware

class LicenseException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


LICENSE_CHECK_LIST = ['mac']

class License():
    def __init__(
        self,
        privateKeyPath: str = None,
        privateKeyPass: str = None,
        publicKeyPath: str = None,
        publicKeyPass: str = None
    ):
        '''
        
        '''
        self.privateKeyPath = privateKeyPath
        self.privateKeyPass = privateKeyPass
        self.publicKeyPath = publicKeyPath
        self.publicKeyPass = publicKeyPass
        self.licensePath = None
        self.private_key = None
        self.public_key = None
        self.license_content = None

        self.load_private_key()
        self.load_public_key()

    def load_private_key(self):
        if not self.privateKeyPath:
            return

        with open(self.privateKeyPath, 'rb') as f:
            self.private_key = RSA.importKey(f.read())

    def load_public_key(self):
        if not self.publicKeyPath:
            return
        
        with open(self.publicKeyPath, 'rb') as f:
            self.public_key = RSA.importKey(f.read())
    
    def install(self, licensePath: str, params=None):
        self.licensePath = licensePath
        if not self.licensePath:
            raise LicenseException('licensePath missing')

        with open(self.licensePath, 'rb') as f:
            text = f.read()
            b = base64.b64decode(text)
            license_data = json.loads(b.decode('utf-8'))
            content = license_data.get('content')
            signature = license_data.get('signature')
            if not content or not signature:
                raise LicenseException('license file invalid')

            b = content.encode('utf-8')
            signature_bin = base64.b64decode(signature)
            # self.public_key.verify(signature_bin, b, hashes.SHA256())
            verifier = PKCS1_signature.new(self.public_key)  # 生成验证信息的类
            digest = SHA256.new()  # 创建一个sha加密的类
            digest.update(b)  # 将获取到的数据进行sha加密
            f = verifier.verify(digest, signature_bin)  # 对数据进行验证，返回bool值
            if f is not True:
                raise LicenseException('证书验证失败，请重新申请证书')
            self.license_content = json.loads(content)
            self.verify(params)

    def init_license_data(self, subject, consumerType, consumerAmount, description, issuedTime: str, expiryTime: str, params: dict = None):
        data = {
            'subject': subject,
            'consumerType': consumerType,
            'consumerAmount': consumerAmount,
            'description': description,
            'validation': {}
        }
        params = params or {}

        if issuedTime is not None:
            if not re.match('\d{4}-\d{2}-\d{2}', issuedTime):
                raise LicenseException('issuedTime 格式错误，比如： 2022-01-01')
            data['issuedTime'] = issuedTime
        if expiryTime is not None:
            data['expiryTime'] = expiryTime
            if not re.match('\d{4}-\d{2}-\d{2}', expiryTime):
                raise LicenseException('expiryTime 格式错误，比如： 2022-01-01')
        
        for k, _ in params.items():
            data['validation'][k] = params[k]

        return data

    def create(self, subject: str = 'castiron', consumerType: str = 'customer', consumerAmount: int = 1, description: str = '', issuedTime: str = None, expiryTime: str = None, params: dict = None, output: str = None):
        if not self.private_key:
            raise LicenseException('private store info not found')  
        data = self.init_license_data(subject, consumerType, consumerAmount, description, issuedTime, expiryTime, params)
        s = json.dumps(data)
        b = s.encode('utf-8')
        signer = PKCS1_signature.new(self.private_key)
        digest = SHA256.new()
        digest.update(b)
        signature = signer.sign(digest)
        base_code = base64.b64encode(signature).decode()
        license_data = {
            'content': s,
            'signature': base_code
        }
        r = json.dumps(license_data).encode('utf-8')
        c = base64.b64encode(r)
        if output is not None:
            with open(output, 'wb') as f1:
                f1.write(c)
        
        return c

    def verify(self, params: dict = None):
        if not self.license_content:
            raise LicenseException('not found license')
        hardware_info = Hardware.get_info()
        params = params or {}
        params = {**hardware_info, **params}
        issuedTime = self.license_content.get('issuedTime')
        expiryTime = self.license_content.get('expiryTime')
        now  = datetime.now()
        if issuedTime:
            issuedDate = datetime.strptime(issuedTime, "%Y-%m-%d")
            if now < issuedDate:
                raise LicenseException(f'证书还未到生效时间: {issuedTime} ~ {expiryTime}')

        if expiryTime:
            expiryDate = datetime.strptime(expiryTime, "%Y-%m-%d")
            if now > expiryDate:
                raise LicenseException(f'证书已过期: {issuedTime} ~ {expiryTime}，请重新申请证书！')
        
        params = params or {}
        if self.license_content.get('validation'):
            validation = self.license_content['validation']
            for k, v in validation.items():
                if params.get(k) != v:
                    errmsg = f'证书 {k} 不匹配，请重新申请证书！'
                    raise LicenseException(errmsg)
            

if __name__ == '__main__':
    license = License(
        privateKeyPath='keystores/private.pem'
    )

    hardware_info = Hardware.get_info()
    r = license.create(
        subject='test',
        issuedTime='2022-10-01',
        expiryTime='2023-10-03',
        params = {
            'mac': hardware_info.get('mac')
        },
        output='license.lic'
    )

    ######

    license = License(
        publicKeyPath='keystores/public.pem',
    )

    license.install('./license.lic')
    print('证书安装成功')
